@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * audiohandler.c
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
  *
  * audiohandler.c is free software.
  *
@@ -302,6 +302,10 @@ int ghb_get_copy_mask(GhbValue *settings)
 {
     gint mask = 0;
 
+    if (ghb_dict_get_bool(settings, "AudioAllowMP2Pass"))
+    {
+        mask |= HB_ACODEC_MP2_PASS;
+    }
     if (ghb_dict_get_bool(settings, "AudioAllowMP3Pass"))
     {
         mask |= HB_ACODEC_MP3_PASS;
