@@ -1,13 +1,12 @@
 /*
  * ghbcompat.h
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
  *
  * ghbcompat.h is free software.
  *
  * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * GNU General Public License version 2, as published by the Free Software
+ * Foundation.
  *
  * ghbcompat.h is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +33,31 @@
 #define GHB_UNSAFE_FILENAME_CHARS "/"
 #endif
 
+#if GTK_CHECK_VERSION(3, 90, 0)
+#define GHB_ICON_SIZE_BUTTON GTK_ICON_SIZE_NORMAL
+#else
+#define GHB_ICON_SIZE_BUTTON GTK_ICON_SIZE_BUTTON
+#endif
+
+#if !GTK_CHECK_VERSION(3, 10, 0)
+#define gtk_image_set_from_icon_name gtk_image_set_from_stock
+#define GHB_PLAY_ICON "gtk-media-play"
+#define GHB_PAUSE_ICON "gtk-media-pause"
+#else
+#define GHB_PLAY_ICON "media-playback-start"
+#define GHB_PAUSE_ICON "media-playback-pause"
+#endif
+
+#if !GTK_CHECK_VERSION(3, 10, 0)
+#define GHB_STOCK_OPEN      GTK_STOCK_OPEN
+#define GHB_STOCK_CANCEL    GTK_STOCK_CANCEL
+#define GHB_STOCK_SAVE      GTK_STOCK_SAVE
+#else
+#define GHB_STOCK_OPEN      "_Open"
+#define GHB_STOCK_CANCEL    "_Cancel"
+#define GHB_STOCK_SAVE      "_Save"
+#endif
+
 static inline void ghb_widget_get_preferred_width(
     GtkWidget *widget, gint *min_width, gint * natural_width)
 {
@@ -103,10 +127,13 @@ static inline void ghb_get_expand_fill(GtkBox * box, GtkWidget * child,
     }
 }
 
-static inline void ghb_box_pack_start(GtkBox * box, GtkWidget * child)
+static inline void ghb_box_append_child(GtkBox * box, GtkWidget * child)
 {
 #if GTK_CHECK_VERSION(3, 90, 0)
-    gtk_box_pack_start(box, child);
+    GtkWidget * sibling = NULL;
+
+    sibling = gtk_widget_get_last_child(GTK_WIDGET(box));
+    gtk_box_insert_child_after(box, child, sibling);
 #else
     gboolean expand, fill;
 
@@ -177,28 +204,71 @@ static inline PangoFontDescription* ghb_widget_get_font(GtkWidget *widget)
     return font;
 }
 
-static inline void ghb_monitor_get_size(GdkWindow *window, gint *w, gint *h)
+#if GTK_CHECK_VERSION(3, 90, 0)
+typedef GdkSurface      GhbSurface;
+typedef GdkSurfaceHints GhbSurfaceHints;
+
+static inline GhbSurface * ghb_widget_get_surface(GtkWidget * widget)
+{
+    return gtk_widget_get_surface(widget);
+}
+
+static inline GdkMonitor *
+ghb_display_get_monitor_at_surface(GdkDisplay * display, GhbSurface * surface)
+{
+    return gdk_display_get_monitor_at_surface(display, surface);
+}
+
+static inline void
+ghb_surface_set_geometry_hints(GhbSurface * surface,
+                               const GdkGeometry * geometry,
+                               GhbSurfaceHints geom_mask)
+{
+    gdk_surface_set_geometry_hints(surface, geometry, geom_mask);
+}
+#else
+typedef GdkWindow       GhbSurface;
+typedef GdkWindowHints  GhbSurfaceHints;
+
+static inline GhbSurface * ghb_widget_get_surface(GtkWidget * widget)
+{
+    return gtk_widget_get_window(widget);
+}
+
+static inline GdkMonitor *
+ghb_display_get_monitor_at_surface(GdkDisplay * display, GhbSurface * surface)
+{
+    return gdk_display_get_monitor_at_window(display, surface);
+}
+
+static inline void
+ghb_surface_set_geometry_hints(GhbSurface * surface,
+                               const GdkGeometry * geometry,
+                               GhbSurfaceHints geom_mask)
+{
+    gdk_window_set_geometry_hints(surface, geometry, geom_mask);
+}
+#endif
+
+static inline void ghb_monitor_get_size(GtkWidget *widget, gint *w, gint *h)
 {
     *w = *h = 0;
 
 #if GTK_CHECK_VERSION(3, 22, 0)
-    if (window != NULL)
+    GdkDisplay * display = gtk_widget_get_display(widget);
+    GhbSurface * surface = ghb_widget_get_surface(widget);
+    if (surface != NULL && display != NULL)
     {
-        GdkMonitor *mm;
-        GdkDisplay *dd;
+        GdkMonitor * monitor;
 
-        dd = gdk_display_get_default();
-        if (dd != NULL)
+        monitor = ghb_display_get_monitor_at_surface(display, surface);
+        if (monitor != NULL)
         {
-            mm = gdk_display_get_monitor_at_window(dd, window);
-            if (mm != NULL)
-            {
-                GdkRectangle rr;
+            GdkRectangle rect;
 
-                gdk_monitor_get_geometry(mm, &rr);
-                *w = rr.width;
-                *h = rr.height;
-            }
+            gdk_monitor_get_geometry(monitor, &rect);
+            *w = rect.width;
+            *h = rect.height;
         }
     }
 #else
@@ -235,23 +305,114 @@ static inline gboolean ghb_strv_contains(const char ** strv, const char * str)
 #endif
 }
 
-#if !GTK_CHECK_VERSION(3, 10, 0)
-#define gtk_image_set_from_icon_name gtk_image_set_from_stock
-#define GHB_PLAY_ICON "gtk-media-play"
-#define GHB_PAUSE_ICON "gtk-media-pause"
+#if GTK_CHECK_VERSION(3, 90, 0)
+
+#define ghb_editable_get_text(e) gtk_editable_get_text(GTK_EDITABLE(e))
+#define ghb_editable_set_text(e,t) gtk_editable_set_text(GTK_EDITABLE(e), (t))
+
 #else
-#define GHB_PLAY_ICON "media-playback-start"
-#define GHB_PAUSE_ICON "media-playback-pause"
+
+#define ghb_editable_get_text(e) gtk_entry_get_text(GTK_ENTRY(e))
+#define ghb_editable_set_text(e,t) gtk_entry_set_text(GTK_ENTRY(e), (t))
+
 #endif
 
-#if !GTK_CHECK_VERSION(3, 10, 0)
-#define GHB_STOCK_OPEN      GTK_STOCK_OPEN
-#define GHB_STOCK_CANCEL    GTK_STOCK_CANCEL
-#define GHB_STOCK_SAVE      GTK_STOCK_SAVE
+#if GTK_CHECK_VERSION(3, 90, 0)
+static inline void
+ghb_image_set_from_icon_name(GtkImage * image, const gchar * name,
+                             GtkIconSize size)
+{
+    gtk_image_set_from_icon_name(image, name);
+    gtk_image_set_icon_size(image, size);
+}
+
+static inline GtkWidget *
+ghb_image_new_from_icon_name(const gchar * name, GtkIconSize size)
+{
+    GtkWidget * image;
+
+    image = gtk_image_new_from_icon_name(name);
+    gtk_image_set_icon_size(GTK_IMAGE(image), size);
+
+    return image;
+}
+
+static inline GtkWidget *
+ghb_button_new_from_icon_name(const gchar * name)
+{
+    return gtk_button_new_from_icon_name(name);
+}
+
+static inline GtkWidget *
+ghb_scale_button_new(gdouble min, gdouble max, gdouble step,
+                     const gchar ** icons)
+{
+    return gtk_scale_button_new(min, max, step, icons);
+}
 #else
-#define GHB_STOCK_OPEN      "_Open"
-#define GHB_STOCK_CANCEL    "_Cancel"
-#define GHB_STOCK_SAVE      "_Save"
+static inline void
+ghb_image_set_from_icon_name(GtkImage * image, const gchar * name,
+                             GtkIconSize size)
+{
+    gtk_image_set_from_icon_name(image, name, size);
+}
+
+static inline GtkWidget *
+ghb_image_new_from_icon_name(const gchar * name, GtkIconSize size)
+{
+    return gtk_image_new_from_icon_name(name, size);
+}
+
+static inline GtkWidget *
+ghb_button_new_from_icon_name(const gchar * name)
+{
+    return gtk_button_new_from_icon_name(name, GHB_ICON_SIZE_BUTTON);
+}
+
+static inline GtkWidget *
+ghb_scale_button_new(gdouble min, gdouble max, gdouble step,
+                     const gchar ** icons)
+{
+    return gtk_scale_button_new(GHB_ICON_SIZE_BUTTON, min, max, step, icons);
+}
+#endif
+
+#if GTK_CHECK_VERSION(3, 90, 0)
+static inline void ghb_drag_status(GdkDrop * ctx, GdkDragAction action,
+                                   guint32 time)
+{
+    gdk_drop_status(ctx, action);
+}
+#else
+static inline void ghb_drag_status(GdkDragContext * ctx, GdkDragAction action,
+                                   guint32 time)
+{
+    gdk_drag_status(ctx, action, time);
+}
+#endif
+
+#if GTK_CHECK_VERSION(3, 90, 0)
+static inline void ghb_entry_set_width_chars(GtkEntry * entry, gint n_chars)
+{
+    gtk_editable_set_width_chars(GTK_EDITABLE(entry), n_chars);
+}
+#else
+static inline void ghb_entry_set_width_chars(GtkEntry * entry, gint n_chars)
+{
+    gtk_entry_set_width_chars(entry, n_chars);
+}
+#endif
+
+#if GTK_CHECK_VERSION(3, 90, 0)
+static inline GdkAtom ghb_atom_string(const char * str)
+{
+    return g_intern_static_string(str);
+}
+#else
+static inline GdkAtom ghb_atom_string(const char * str)
+{
+    return gdk_atom_intern_static_string(str);
+}
 #endif
 
 #endif // _GHB_COMPAT_H_
