@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * presets.c
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
  *
  * presets.c is free software.
  *
@@ -256,17 +256,51 @@ ghb_preset_to_settings(GhbValue *settings, GhbValue *preset)
     ghb_dict_set(settings, "scale_width", ghb_value_dup(
         ghb_dict_get_value(settings, "PictureWidth")));
 
-    int width, height, uses_pic, autoscale;
+    if (ghb_dict_get_bool(settings, "PictureAutoCrop"))
+    {
+        ghb_dict_set_string(settings, "crop_mode", "auto");
+    }
+    else
+    {
+        int ct = ghb_dict_get_int(settings, "PictureTopCrop");
+        int cb = ghb_dict_get_int(settings, "PictureBottomCrop");
+        int cl = ghb_dict_get_int(settings, "PictureLeftCrop");
+        int cr = ghb_dict_get_int(settings, "PictureRightCrop");
+        if (ct == 0 && cb == 0 && cl == 0 && cr == 0)
+        {
+            ghb_dict_set_string(settings, "crop_mode", "none");
+        }
+        else
+        {
+            ghb_dict_set_string(settings, "crop_mode", "custom");
+        }
+    }
+
+    int width, height;
+    const gchar * resolution_limit;
 
     width    = ghb_dict_get_int(settings, "PictureWidth");
     height   = ghb_dict_get_int(settings, "PictureHeight");
-    uses_pic = ghb_dict_get_int(settings, "UsesPictureSettings");
+    resolution_limit = ghb_lookup_resolution_limit(width, height);
+    ghb_dict_set_string(settings, "resolution_limit", resolution_limit);
 
-    autoscale = uses_pic != 1 || (width == 0 && height == 0);
-    ghb_dict_set_bool(settings, "autoscale", autoscale);
-    ghb_dict_set_int(settings, "PictureWidthEnable", !autoscale);
-    ghb_dict_set_int(settings, "PictureHeightEnable", !autoscale);
+    const char * rotate = ghb_dict_get_string(settings, "PictureRotate");
+    if (rotate != NULL)
+    {
+        int         angle, hflip;
+        char      * angle_opt;
+        hb_dict_t * filter_settings;
 
+        filter_settings = hb_generate_filter_settings(HB_FILTER_ROTATE,
+                                                      NULL, NULL, rotate);
+        angle = hb_dict_get_int(filter_settings, "angle");
+        hflip = hb_dict_get_bool(filter_settings, "hflip");
+        angle_opt = g_strdup_printf("%d", angle);
+        ghb_dict_set_string(settings, "rotate", angle_opt);
+        ghb_dict_set_int(settings, "hflip", hflip);
+        g_free(angle_opt);
+        hb_value_free(&filter_settings);
+    }
     gint vqtype;
 
     vqtype = ghb_dict_get_int(settings, "VideoQualityType");
@@ -396,6 +430,9 @@ ghb_preset_to_settings(GhbValue *settings, GhbValue *preset)
             {
                 default:
                     break;
+                case HB_ACODEC_MP2_PASS:
+                    ghb_dict_set_bool(settings, "AudioAllowMP2Pass", 1);
+                    break;
                 case HB_ACODEC_LAME:
                 case HB_ACODEC_MP3_PASS:
                     ghb_dict_set_bool(settings, "AudioAllowMP3Pass", 1);
@@ -437,7 +474,7 @@ ghb_preset_to_settings(GhbValue *settings, GhbValue *preset)
 }
 
 // Initialization order of some widgets matter because the value of
-// these widgets are used to establich limits on the values that
+// these widgets are used to establish limits on the values that
 // other widgets are allowed to take.
 //
 // So make sure these get initialized first.
@@ -474,7 +511,7 @@ ghb_settings_to_ui(signal_user_data_t *ud, GhbValue *dict)
     }
 
     iter = ghb_dict_iter_init(tmp);
-    // middle (void*) cast prevents gcc warning "defreferencing type-punned
+    // middle (void*) cast prevents gcc warning "dereferencing type-punned
     // pointer will break strict-aliasing rules"
     while (ghb_dict_iter_next(tmp, &iter, &key, &gval))
     {
@@ -1649,6 +1686,10 @@ ghb_remove_old_queue_file(int pid)
 GhbValue* ghb_create_copy_mask(GhbValue *settings)
 {
     GhbValue *copy_mask = ghb_array_new();
+    if (ghb_dict_get_bool(settings, "AudioAllowMP2Pass"))
+    {
+        ghb_array_append(copy_mask, ghb_string_value_new("copy:mp2"));
+    }
     if (ghb_dict_get_bool(settings, "AudioAllowMP3Pass"))
     {
         ghb_array_append(copy_mask, ghb_string_value_new("copy:mp3"));
@@ -1690,31 +1731,24 @@ ghb_settings_to_preset(GhbValue *settings)
 {
     GhbValue *preset = ghb_value_dup(settings);
 
-    gboolean autoscale, br, constant;
+    gboolean br, constant;
 
     ghb_dict_remove(preset, "title");
     ghb_dict_set_bool(preset, "Default", 0);
-    if (!ghb_dict_get_bool(preset, "PictureWidthEnable"))
-    {
-        ghb_dict_remove(preset, "PictureWidth");
-    }
-    if (!ghb_dict_get_bool(preset, "PictureHeightEnable"))
-    {
-        ghb_dict_remove(preset, "PictureHeight");
-    }
-    autoscale = !ghb_dict_get_bool(preset, "PictureWidthEnable") &&
-                !ghb_dict_get_bool(preset, "PictureHeightEnable");
 
     br = ghb_dict_get_bool(preset, "vquality_type_bitrate");
     constant = ghb_dict_get_bool(preset, "vquality_type_constant");
-    if (autoscale)
-    {
-        ghb_dict_set_int(preset, "UsesPictureSettings", 2);
-    }
-    else
-    {
-        ghb_dict_set_int(preset, "UsesPictureSettings", 1);
-    }
+
+    const char * angle    = ghb_dict_get_string(preset, "rotate");
+    int          hflip    = ghb_dict_get_int(preset, "hflip");
+    char       * rot_flip = g_strdup_printf("angle=%s:hflip=%d", angle, hflip);
+    ghb_dict_set_string(preset, "PictureRotate", rot_flip);
+    g_free(rot_flip);
+
+    const gchar * crop_mode;
+
+    crop_mode = ghb_dict_get_string(settings, "crop_mode");
+    ghb_dict_set_bool(preset, "PictureAutoCrop", !strcmp(crop_mode, "auto"));
 
     // VideoQualityType/0/1/2 - vquality_type_/target/bitrate/constant
     // *note: target is no longer used
@@ -1768,14 +1802,18 @@ ghb_settings_to_preset(GhbValue *settings)
         g_string_append_printf(str, "%s", tune);
         sep = ",";
     }
-    if (ghb_dict_get_bool(preset, "x264FastDecode"))
+    int encoder = ghb_get_video_encoder(settings);
+    if (encoder & HB_VCODEC_X264_MASK)
     {
-        g_string_append_printf(str, "%s%s", sep, "fastdecode");
-        sep = ",";
-    }
-    if (ghb_dict_get_bool(preset, "x264ZeroLatency"))
-    {
-        g_string_append_printf(str, "%s%s", sep, "zerolatency");
+        if (ghb_dict_get_bool(preset, "x264FastDecode"))
+        {
+            g_string_append_printf(str, "%s%s", sep, "fastdecode");
+            sep = ",";
+        }
+        if (ghb_dict_get_bool(preset, "x264ZeroLatency"))
+        {
+            g_string_append_printf(str, "%s%s", sep, "zerolatency");
+        }
     }
     char *tunes;
     tunes = g_string_free(str, FALSE);
@@ -1875,7 +1913,7 @@ ghb_presets_load(signal_user_data_t *ud)
     {
         if (presets_add_config_file("presets") < 0)
         {
-            ghb_log("Failed to read presets file, initailizing new presets...");
+            ghb_log("Failed to read presets file, initializing new presets...");
             hb_presets_builtin_update();
             store_presets();
         }
@@ -2219,8 +2257,6 @@ static void preset_save_action(signal_user_data_t *ud, gboolean as)
     const gchar       * fullname;
     int                 type;
     hb_preset_index_t * path;
-    int                 width, height;
-    gboolean            autoscale;
     GtkWidget         * dialog;
     GtkWidget         * widget;
     GtkEntry          * entry;
@@ -2231,13 +2267,8 @@ static void preset_save_action(signal_user_data_t *ud, gboolean as)
     name      = ghb_dict_get_string(ud->settings, "PresetName");
     type      = ghb_dict_get_int(ud->settings, "Type");
     fullname  = ghb_dict_get_string(ud->settings, "PresetFullName");
-    width     = ghb_dict_get_int(ud->settings, "PictureWidth");
-    height    = ghb_dict_get_int(ud->settings, "PictureHeight");
-    autoscale = ghb_dict_get_bool(ud->settings, "autoscale");
 
     ghb_ui_update(ud, "PresetSetDefault", ghb_boolean_value(FALSE));
-    ghb_ui_update(ud, "PictureWidthEnable", ghb_boolean_value(!autoscale));
-    ghb_ui_update(ud, "PictureHeightEnable", ghb_boolean_value(!autoscale));
 
     path = hb_preset_search_index(fullname, 0, type);
 
@@ -2281,16 +2312,6 @@ static void preset_save_action(signal_user_data_t *ud, gboolean as)
     }
     ghb_ui_update(ud, "PresetCategory", ghb_string_value(category));
     tv = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "PresetDescription"));
-    if (!width)
-    {
-        width = ghb_dict_get_int(ud->settings, "scale_width");
-        ghb_ui_update(ud, "PictureWidth", ghb_int_value(width));
-    }
-    if (!height)
-    {
-        height = ghb_dict_get_int(ud->settings, "scale_height");
-        ghb_ui_update(ud, "PictureHeight", ghb_int_value(height));
-    }
 
     dialog   = GHB_WIDGET(ud->builder, "preset_save_dialog");
     entry    = GTK_ENTRY(GHB_WIDGET(ud->builder, "PresetName"));
@@ -2401,18 +2422,32 @@ G_MODULE_EXPORT void
 preset_remove_action_cb(GSimpleAction *action, GVariant *param,
                         signal_user_data_t *ud)
 {
-    int                 type;
-    const char        * fullname;
     hb_preset_index_t * path;
+    GhbValue          * preset;
 
-    type     = ghb_dict_get_int(ud->settings, "Type");
-    fullname = ghb_dict_get_string(ud->settings, "PresetFullName");
-    if (fullname == NULL)
+    path = get_selected_path(ud);
+    if (path != NULL)
     {
-        return;
+        preset = hb_preset_get(path);
     }
-    path = hb_preset_search_index(fullname, 0, type);
-    if (path == NULL)
+    if (path == NULL || preset == NULL)
+    {
+        const char * fullname;
+        int          type;
+
+        fullname = ghb_dict_get_string(ud->settings, "PresetFullName");
+        if (fullname == NULL)
+        {
+            return;
+        }
+        type     = ghb_dict_get_int(ud->settings, "Type");
+        path = hb_preset_search_index(fullname, 0, type);
+        if (path != NULL)
+        {
+            preset = hb_preset_get(path);
+        }
+    }
+    if (path == NULL || preset == NULL)
     {
         return;
     }
@@ -2423,7 +2458,7 @@ preset_remove_action_cb(GSimpleAction *action, GVariant *param,
     GtkResponseType   response;
     const char      * name;
 
-    name  = ghb_dict_get_string(ud->settings, "PresetName");
+    name  = ghb_dict_get_string(preset, "PresetName");
     is_folder = preset_is_folder(path);
     hb_window = GTK_WINDOW(GHB_WIDGET(ud->builder, "hb_window"));
     dialog = gtk_message_dialog_new(hb_window, GTK_DIALOG_MODAL,
@@ -2435,42 +2470,81 @@ preset_remove_action_cb(GSimpleAction *action, GVariant *param,
     gtk_widget_destroy(dialog);
     if (response == GTK_RESPONSE_YES)
     {
-        GtkTreeView      * treeview;
-        GtkTreeSelection * selection;
-        gboolean           valid = TRUE;
-        hb_value_t       * preset;
-
+        int depth = path->depth;
 
         // Determine which preset to highlight after deletion done
         hb_preset_index_t new_path = *path;
+        // Always select a preset, not a folder
+        if (depth == 1)
+        {
+            new_path.depth = 2;
+        }
         // Try next
-        new_path.index[path->depth - 1] = path->index[path->depth - 1] + 1;
+        new_path.index[depth - 1] = path->index[depth - 1] + 1;
         preset = hb_preset_get(&new_path);
-        if (preset == NULL)
+        // After deletion, index of new selected item is one less
+        new_path.index[depth - 1]--;
+        if (preset == NULL && path->index[depth - 1] > 0)
         {
             // Try previous
-            new_path.index[path->depth - 1] =
-                path->index[path->depth - 1] - 1;
+            new_path.index[depth - 1] = path->index[depth - 1] - 1;
             preset = hb_preset_get(&new_path);
-            if (preset == NULL)
+        }
+        if (preset == NULL)
+        {
+            // perhaps we are deleting the last item in a folder
+            // Try first item in next and previous folders
+            depth = 1;
+            new_path.index[1] = 0;
+            // Try next
+            new_path.index[depth - 1] = path->index[depth - 1] + 1;
+            preset = hb_preset_get(&new_path);
+            // After deletion, index of new selected item is one less
+            new_path.index[depth - 1]--;
+            if (preset == NULL && path->index[depth - 1] > 0)
             {
-                valid = FALSE;
+                // Try previous
+                new_path.index[depth - 1] = path->index[depth - 1] - 1;
+                preset = hb_preset_get(&new_path);
             }
         }
 
-        // Remove the selected item
-        // First unselect it so that selecting the new item works properly
+        GtkTreeView      * treeview;
+        GtkTreeSelection * selection;
+
+        // unselect item to be deleted
         treeview  = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "presets_list"));
         selection = gtk_tree_view_get_selection(treeview);
         gtk_tree_selection_unselect_all(selection);
 
+        // Remove the item
         if (hb_preset_delete(path) >= 0)
         {
-            store_presets();
             presets_list_remove(ud, path);
+            if (path->depth == 2)
+            {
+                // If deleting this item resulted in an empty folder
+                // delete the folder
+                int count = 0;
+                hb_value_t * folder;
+
+                path->depth = 1;
+                folder = hb_presets_get_folder_children(path);
+                if (folder != NULL)
+                {
+                    count = ghb_array_len(folder);
+                }
+                if (count == 0)
+                {
+                    // delete the folder
+                    hb_preset_delete(path);
+                    presets_list_remove(ud, path);
+                }
+            }
+            store_presets();
             ghb_presets_menu_reinit(ud);
         }
-        if (valid)
+        if (preset != NULL)
         {
             select_preset2(ud, &new_path);
         }
