@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * videohandler.c
- * Copyright (C) John Stebbins 2008-2019 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2022 <stebbins@stebbins>
  *
  * videohandler.c is free software.
  *
@@ -163,7 +163,7 @@ update_adv_settings_tooltip(signal_user_data_t *ud)
 
         if (w == 0 || h == 0)
         {
-            if (!ghb_dict_get_bool(ud->settings, "autoscale"))
+            if (!ghb_dict_get_bool(ud->settings, "PictureUseMaximumSize"))
             {
                 w = ghb_dict_get_int(ud->settings, "PictureWidth");
                 h = ghb_dict_get_int(ud->settings, "PictureHeight");
@@ -289,7 +289,7 @@ format_video_preset_cb(GtkScale *scale, gdouble val, signal_user_data_t *ud)
         // When the range of a slider changes, GTK used to sample all the
         // possible values to determine the correct amount of screen space to
         // allocate for the value strings. Some *genius* decided it would be
-        // more effecient to just sample the first and last value which means
+        // more efficient to just sample the first and last value which means
         // that if certain characters are wider than others and the middle
         // values happen to use those characters, the space allocated is
         // too small and the string wraps to the next line or is truncated.
@@ -328,7 +328,7 @@ format_vquality_cb(GtkScale *scale, gdouble val, signal_user_data_t *ud)
             // When the range of a slider changes, GTK used to sample all the
             // possible values to determine the correct amount of screen space to
             // allocate for the value strings. Some *genius* decided it would be
-            // more effecient to just sample the first and last value which means
+            // more efficient to just sample the first and last value which means
             // that if certain characters are wider than others and the middle
             // values happen to use those characters, the space allocated is
             // too small and the string wraps to the next line or is truncated.
@@ -357,7 +357,7 @@ format_vquality_cb(GtkScale *scale, gdouble val, signal_user_data_t *ud)
             // When the range of a slider changes, GTK used to sample all the
             // possible values to determine the correct amount of screen space to
             // allocate for the value strings. Some *genius* decided it would be
-            // more effecient to just sample the first and last value which means
+            // more efficient to just sample the first and last value which means
             // that if certain characters are wider than others and the middle
             // values happen to use those characters, the space allocated is
             // too small and the string wraps to the next line or is truncated.
