@@ -1,6 +1,6 @@
 /* dvd.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -205,7 +205,8 @@ static void add_subtitle( hb_list_t * list_subtitle, int position,
     subtitle->config.dest    = RENDERSUB;
     subtitle->stream_type    = 0xbd;
     subtitle->substream_type = 0x20 + position;
-    subtitle->codec          = WORK_DECVOBSUB;
+    subtitle->codec          = WORK_DECAVSUB;
+    subtitle->codec_param    = AV_CODEC_ID_DVD_SUBTITLE;
     subtitle->timebase.num   = 1;
     subtitle->timebase.den   = 90000;
 
@@ -516,6 +517,7 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
         }
         if( !audio->id )
         {
+            free(audio);
             continue;
         }
 
@@ -861,6 +863,11 @@ static int hb_dvdread_seek( hb_dvd_t * e, float f )
 
     count = f * d->title_block_count;
 
+    if (d->file == NULL)
+    {
+        return 1;
+    }
+
     for( i = d->cell_start; i <= d->cell_end; i++ )
     {
         sizeCell = d->pgc->cell_playback[i].last_sector + 1 -
@@ -1454,4 +1461,3 @@ void hb_dvd_set_dvdnav( int enable )
     else
         dvd_methods = hb_dvdread_methods();
 }
-
