@@ -1,6 +1,6 @@
 /* dvdnav.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -351,7 +351,8 @@ static void add_subtitle( hb_list_t * list_subtitle, int position,
     subtitle->config.dest    = RENDERSUB;
     subtitle->stream_type    = 0xbd;
     subtitle->substream_type = 0x20 + position;
-    subtitle->codec          = WORK_DECVOBSUB;
+    subtitle->codec          = WORK_DECAVSUB;
+    subtitle->codec_param    = AV_CODEC_ID_DVD_SUBTITLE;
     subtitle->timebase.num   = 1;
     subtitle->timebase.den   = 90000;
 
@@ -1748,6 +1749,11 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
                     hb_deep_log(2, "dvdnav: vts change, found next title");
+                    if (error_count > 0)
+                    {
+                        // Last read attempt failed
+                        hb_set_work_error(d->h, HB_ERROR_READ);
+                    }
                     return NULL;
                 }
             }
@@ -1772,6 +1778,11 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
                     hb_deep_log(2, "dvdnav: cell change, found next title");
+                    if (error_count > 0)
+                    {
+                        // Last read attempt failed
+                        hb_set_work_error(d->h, HB_ERROR_READ);
+                    }
                     return NULL;
                 }
                 c = FindChapterIndex(d->list_dvd_chapter, pgcn, pgn);
@@ -1783,6 +1794,11 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                         // a transition to an earlier chapter means we're done.
                         hb_buffer_close( &b );
                         hb_deep_log(2, "dvdnav: cell change, previous chapter");
+                        if (error_count > 0)
+                        {
+                            // Last read attempt failed
+                            hb_set_work_error(d->h, HB_ERROR_READ);
+                        }
                         return NULL;
                     }
                     chapter = d->chapter = c;
@@ -1791,6 +1807,11 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 {
                     hb_buffer_close( &b );
                     hb_deep_log(2, "dvdnav: cell change, previous cell");
+                    if (error_count > 0)
+                    {
+                        // Last read attempt failed
+                        hb_set_work_error(d->h, HB_ERROR_READ);
+                    }
                     return NULL;
                 }
                 d->cell = cell_event->cellN;
@@ -1829,6 +1850,11 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
             d->stopped = 1;
             hb_buffer_close( &b );
             hb_deep_log(2, "dvdnav: stop");
+            if (error_count > 0)
+            {
+                // Last read attempt failed
+                hb_set_work_error(d->h, HB_ERROR_READ);
+            }
             return NULL;
 
         default:
@@ -2176,4 +2202,4 @@ static void TitleCloseIfo(hb_dvdnav_t * d)
     d->vts   = 0;
     d->pgcn  = 0;
     d->pgn   = 0;
-}
+}
\ No newline at end of file
