@@ -1,6 +1,6 @@
 /* encavcodecaudio.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -14,6 +14,7 @@ struct hb_work_private_s
 {
     hb_job_t       * job;
     AVCodecContext * context;
+    AVPacket       * pkt;
 
     int              out_discrete_channels;
     int              samples_per_frame;
@@ -52,6 +53,13 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
     pv->job               = job;
     pv->list              = hb_list_init();
     pv->last_pts          = AV_NOPTS_VALUE;
+    pv->pkt               = av_packet_alloc();
+
+    if (pv->pkt == NULL)
+    {
+        hb_error("encavcodecaInit: av_packet_alloc failed");
+        return 1;
+    }
 
     // channel count, layout and matrix encoding
     int matrix_encoding;
@@ -253,7 +261,8 @@ static int encavcodecaInit(hb_work_object_t *w, hb_job_t *job)
                        context->sample_rate, 0);
         av_opt_set_int(pv->swresample, "out_sample_rate",
                        context->sample_rate, 0);
-        if (hb_audio_dither_is_supported(audio->config.out.codec))
+        if (hb_audio_dither_is_supported(audio->config.out.codec,
+                                         audio->config.in.sample_bit_depth))
         {
             // dithering needs the sample rate
             av_opt_set_int(pv->swresample, "dither_method",
@@ -318,6 +327,8 @@ static void encavcodecaClose(hb_work_object_t * w)
             hb_avcodec_free_context(&pv->context);
         }
 
+        av_packet_free(&pv->pkt);
+
         if (pv->output_buf != NULL)
         {
             free(pv->output_buf);
@@ -352,11 +363,9 @@ static void get_packets( hb_work_object_t * w, hb_buffer_list_t * list )
     {
         // Prepare output packet
         int           ret;
-        AVPacket      pkt;
         hb_buffer_t * out;
 
-        av_init_packet(&pkt);
-        ret = avcodec_receive_packet(pv->context, &pkt);
+        ret = avcodec_receive_packet(pv->context, pv->pkt);
         if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
         {
             break;
@@ -367,19 +376,19 @@ static void get_packets( hb_work_object_t * w, hb_buffer_list_t * list )
             break;
         }
 
-        out = hb_buffer_init(pkt.size);
-        memcpy(out->data, pkt.data, out->size);
+        out = hb_buffer_init(pv->pkt->size);
+        memcpy(out->data, pv->pkt->data, out->size);
 
         // FIXME: On windows builds, there is an upstream bug in the lame
         // encoder that causes an extra output packet that has the same
         // timestamp as the second to last packet.  This causes an error
         // during muxing. Work around it by dropping such packets here.
         // See: https://github.com/HandBrake/HandBrake/issues/726
-        if (pkt.pts > pv->last_pts)
+        if (pv->pkt->pts > pv->last_pts)
         {
             // The output pts from libav is in context->time_base. Convert it
             // back to our timebase.
-            out->s.start = av_rescale_q(pkt.pts, pv->context->time_base,
+            out->s.start = av_rescale_q(pv->pkt->pts, pv->context->time_base,
                                         (AVRational){1, 90000});
             out->s.duration  = (double)90000 * pv->samples_per_frame /
                                                audio->config.out.samplerate;
@@ -388,9 +397,9 @@ static void get_packets( hb_work_object_t * w, hb_buffer_list_t * list )
             out->s.frametype = HB_FRAME_AUDIO;
 
             hb_buffer_list_append(list, out);
-            pv->last_pts = pkt.pts;
+            pv->last_pts = pv->pkt->pts;
         }
-        av_packet_unref(&pkt);
+        av_packet_unref(pv->pkt);
     }
 }
 
@@ -409,7 +418,10 @@ static void Encode(hb_work_object_t *w, hb_buffer_list_t *list)
 
         // Prepare input frame
         int     out_size;
-        AVFrame frame = { .nb_samples = pv->samples_per_frame, };
+        AVFrame frame = { .nb_samples = pv->samples_per_frame,
+                          .format = pv->context->sample_fmt,
+                          .channels = pv->context->channels
+        };
 
         out_size = av_samples_get_buffer_size(NULL,
                                               pv->context->channels,
