@@ -1,6 +1,6 @@
 /* encx265.c
 
-   Copyright (c) 2003-2021 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -259,12 +259,24 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
         }
     }
 
+    if (job->chroma_location != AVCHROMA_LOC_UNSPECIFIED) {
+        char chromaLocation[256];
+        snprintf(chromaLocation, sizeof(chromaLocation),
+                 "%d",
+                 job->chroma_location - 1);
+
+        if (param_parse(pv, param, "chromaloc", chromaLocation))
+        {
+            goto fail;
+        }
+    }
+
     /* Bit depth */
     pv->bit_depth = hb_get_bit_depth(job->output_pix_fmt);
 
     /* iterate through x265_opts and parse the options */
     hb_dict_t *x265_opts;
-    int override_mastering = 0, override_coll = 0;
+    int override_mastering = 0, override_coll = 0, override_chroma_location = 0;
     x265_opts = hb_encopts_to_dict(job->encoder_options, job->vcodec);
 
     hb_dict_iter_t iter;
@@ -284,6 +296,10 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
         {
             override_coll = 1;
         }
+        if (!strcmp(key, "chromaloc"))
+        {
+            override_chroma_location = 1;
+        }
 
         // here's where the strings are passed to libx265 for parsing
         // unknown options or bad values are non-fatal, see encx264.c
@@ -342,6 +358,15 @@ int encx265Init(hb_work_object_t *w, hb_job_t *job)
         job->coll.max_cll  = param->maxCLL;
     }
 
+    /*
+     * Reload chroma location settings in case custom
+     * values were set in the encoder_options string.
+     */
+    if (override_chroma_location)
+    {
+        job->chroma_location = param->vui.chromaSampleLocTypeBottomField + 1;
+    }
+
     /*
      * Settings which can't be overridden in the encoder_options string
      * (muxer-specific settings, resolution, ratecontrol, etc.).
