@@ -1,6 +1,6 @@
 /* fifo.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -16,7 +16,7 @@
 #endif
 
 #ifndef SYS_DARWIN
-#if defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
+#if defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined(SYS_NETBSD)
 #include <stdlib.h>
 #else
 #include <malloc.h>
@@ -725,14 +725,15 @@ void hb_buffer_close( hb_buffer_t ** _b )
 #if HB_PROJECT_FEATURE_QSV
         // Reclaim QSV resources before dropping the buffer.
         // when decoding without QSV, the QSV atom will be NULL.
-        if(b->qsv_details.frame)
+        if(b->qsv_details.frame && b->qsv_details.ctx != NULL)
         {
             mfxFrameSurface1 *surface = (mfxFrameSurface1*)b->qsv_details.frame->data[3];
             if(surface)
             {
-                hb_qsv_release_surface_from_pool(surface->Data.MemId);
+                hb_qsv_release_surface_from_pool_by_surface_pointer(b->qsv_details.qsv_frames_ctx, surface);
                 b->qsv_details.frame->data[3] = 0;
             }
+            av_frame_unref(b->qsv_details.frame);
         }
         if (b->qsv_details.qsv_atom != NULL && b->qsv_details.ctx != NULL)
         {
@@ -869,6 +870,9 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
     image->format = buf->f.fmt;
     image->width = buf->f.width;
     image->height = buf->f.height;
+    image->color_prim     = buf->f.color_prim;
+    image->color_transfer = buf->f.color_transfer;
+    image->color_matrix   = buf->f.color_matrix;
     memcpy(image->data, buf->data, buf->size);
 
     int p;
