@@ -1,6 +1,6 @@
 /* internal.h
 
-   Copyright (c) 2003-2021 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -125,6 +125,7 @@ struct hb_image_format_s
     int           color_transfer;
     int           color_matrix;
     int           color_range;
+    int           chroma_location;
     int           max_plane;
     int           window_width;
     int           window_height;
@@ -441,6 +442,7 @@ enum
     WORK_RENDER,
     WORK_ENCAVCODEC,
     WORK_ENCQSV,
+    WORK_ENCVT,
     WORK_ENCX264,
     WORK_ENCX265,
     WORK_ENCTHEORA,
