@@ -1,7 +1,7 @@
 /* nlmeans.h
 
    Copyright (c) 2013 Dirk Farin
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -24,7 +24,8 @@ typedef struct
                            int       dst_w,
                            int       dst_h,
                            int       dx,
-                           int       dy);
+                           int       dy,
+                           int       n);
 } NLMeansFunctions;
 
 void nlmeans_init_x86(NLMeansFunctions *functions);
