@@ -1,6 +1,6 @@
 /* nvenc_common.h
  *
- * Copyright (c) 2003-2021 HandBrake Team
+ * Copyright (c) 2003-2022 HandBrake Team
  * This file is part of the HandBrake source code.
  * Homepage: <http://handbrake.fr/>.
  * It may be used under the terms of the GNU General Public License v2.
@@ -12,6 +12,6 @@
 
 int            hb_nvenc_h264_available();
 int            hb_nvenc_h265_available();
-char *         hb_map_nvenc_preset_name (char * preset);
+char *         hb_map_nvenc_preset_name (const char * preset);
 
 #endif // HANDBRAKE_NVENC_COMMON_H
