@@ -1,6 +1,6 @@
 /* qsv_common.h
  *
- * Copyright (c) 2003-2021 HandBrake Team
+ * Copyright (c) 2003-2022 HandBrake Team
  * This file is part of the HandBrake source code.
  * Homepage: <http://handbrake.fr/>.
  * It may be used under the terms of the GNU General Public License v2.
@@ -23,8 +23,15 @@ void hb_qsv_force_workarounds(); // for developers only
 #ifdef __LIBHB__
 // Private API
 
-#include "mfx/mfxvideo.h"
-#include "mfx/mfxplugin.h"
+#include "vpl/mfxvideo.h"
+
+#define HB_QSV_VERSION_ATLEAST(MAJOR, MINOR)   \
+    (MFX_VERSION_MAJOR > (MAJOR) ||         \
+     MFX_VERSION_MAJOR == (MAJOR) && MFX_VERSION_MINOR >= (MINOR))
+#define HB_QSV_ONEVPL HB_QSV_VERSION_ATLEAST(2, 0)
+#if !HB_QSV_ONEVPL
+    #include "mfx/mfxplugin.h"
+#endif
 #include "handbrake/hb_dict.h"
 #include "handbrake/qsv_libav.h"
 
@@ -93,9 +100,11 @@ int            hb_qsv_get_platform(int adapter_index);
 int            hb_qsv_get_adapter_index();
 int            hb_qsv_implementation_is_hardware(mfxIMPL implementation);
 
+#if !HB_QSV_ONEVPL
 /* Automatically load and unload any required MFX plug-ins */
 hb_list_t* hb_qsv_load_plugins  (int adapter_index, hb_qsv_info_t *info, mfxSession session, mfxVersion version);
 void       hb_qsv_unload_plugins(hb_list_t     **_l,  mfxSession session, mfxVersion version);
+#endif
 
 /* Intel Quick Sync Video DECODE utilities */
 const char* hb_qsv_decode_get_codec_name(enum AVCodecID codec_id);
