@@ -116,7 +116,7 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <stdint.h>
 #include <string.h>
-#include "mfx/mfxvideo.h"
+#include "vpl/mfxvideo.h"
 #include "libavutil/mem.h"
 #include "libavutil/time.h"
 #include "libavcodec/avcodec.h"
@@ -140,6 +140,11 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 // sleep is defined in milliseconds
 #define hb_qsv_sleep(x) av_usleep((x)*1000)
 
+#define HB_QSV_VERSION_ATLEAST(MAJOR, MINOR)   \
+    (MFX_VERSION_MAJOR > (MAJOR) ||         \
+     MFX_VERSION_MAJOR == (MAJOR) && MFX_VERSION_MINOR >= (MINOR))
+#define HB_QSV_ONEVPL HB_QSV_VERSION_ATLEAST(2, 0)
+
 #define HB_QSV_ZERO_MEMORY(VAR)                    {memset(&VAR, 0, sizeof(VAR));}
 #define HB_QSV_ALIGN32(X)                      (((mfxU32)((X)+31)) & (~ (mfxU32)31))
 #define HB_QSV_ALIGN16(value)                  (((value + 15) >> 4) << 4)
@@ -161,7 +166,6 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #define HB_QSV_SURFACE_NUM              80
 #define HB_QSV_SYNC_NUM                 HB_QSV_SURFACE_NUM*3/4
-#define HB_QSV_BUF_SIZE_DEFAULT         4096*2160*10
 #define HB_QSV_JOB_SIZE_DEFAULT         10
 #define HB_QSV_SYNC_TIME_DEFAULT        10000
 // see hb_qsv_get_free_sync, hb_qsv_get_free_surface , 100 if usleep(10*1000)(10ms) == 1 sec
@@ -275,8 +279,9 @@ typedef struct hb_qsv_space {
 
     mfxFrameAllocResponse response;
     mfxFrameAllocRequest request[2];    // [0] - in, [1] - out, if needed
-
+#if !HB_QSV_ONEVPL
     mfxExtOpaqueSurfaceAlloc ext_opaque_alloc;
+#endif
     mfxExtBuffer **p_ext_params;
     uint16_t p_ext_param_num;
 
@@ -370,7 +375,9 @@ typedef struct hb_qsv_alloc_buffer {
 typedef struct hb_qsv_allocators_space {
     hb_qsv_space *space;
     mfxFrameAllocator frame_alloc;
+#if !HB_QSV_ONEVPL
     mfxBufferAllocator buffer_alloc;
+#endif
 } hb_qsv_allocators_space;
 
 typedef struct hb_qsv_config {
