@@ -15,6 +15,10 @@
 #include "handbrake/hbavfilter.h"
 #include "handbrake/avfilter_priv.h"
 
+#if HB_PROJECT_FEATURE_QSV
+#include "handbrake/qsv_common.h"
+#endif
+
 struct hb_avfilter_graph_s
 {
     AVFilterGraph    * avgraph;
@@ -24,10 +28,11 @@ struct hb_avfilter_graph_s
     char             * settings;
     AVFrame          * frame;
     AVRational         out_time_base;
+    hb_job_t         * job;
 };
 
 static AVFilterContext * append_filter( hb_avfilter_graph_t * graph,
-                                        const char * name, const char * args)
+                                        const char * name, const char * args, AVBufferSrcParameters *par)
 {
     AVFilterContext * filter;
     int               result;
@@ -38,6 +43,16 @@ static AVFilterContext * append_filter( hb_avfilter_graph_t * graph,
     {
         return NULL;
     }
+
+    if (par)
+    {
+        result = av_buffersrc_parameters_set(filter, par);
+        if (result < 0)
+        {
+            return NULL;
+        }
+    }
+
     if (graph->last != NULL)
     {
         result = avfilter_link(graph->last, 0, filter, 0);
@@ -55,12 +70,13 @@ static AVFilterContext * append_filter( hb_avfilter_graph_t * graph,
 hb_avfilter_graph_t *
 hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
 {
-    hb_avfilter_graph_t * graph;
-    AVFilterInOut       * in = NULL, * out = NULL;
-    AVFilterContext     * avfilter;
-    char                * settings_str;
-    int                   result;
-    char                * filter_args;
+    hb_avfilter_graph_t   * graph;
+    AVFilterInOut         * in = NULL, * out = NULL;
+    AVBufferSrcParameters * par = NULL;
+    AVFilterContext       * avfilter;
+    char                  * settings_str;
+    int                     result;
+    char                  * filter_args;
 
     graph = calloc(1, sizeof(hb_avfilter_graph_t));
     if (graph == NULL)
@@ -75,6 +91,7 @@ hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
         goto fail;
     }
 
+    graph->job = init->job;
     graph->settings = settings_str;
     graph->avgraph = avfilter_graph_alloc();
     if (graph->avgraph == NULL)
@@ -82,9 +99,6 @@ hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
         hb_error("hb_avfilter_graph_init: avfilter_graph_alloc failed");
         goto fail;
     }
-
-    av_opt_set(graph->avgraph, "scale_sws_opts", "lanczos+accurate_rnd", 0);
-
     result = avfilter_graph_parse2(graph->avgraph, settings_str, &in, &out);
     if (result < 0 || in == NULL || out == NULL)
     {
@@ -94,15 +108,57 @@ hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
     }
 
     // Build filter input
-    filter_args = hb_strdup_printf(
-                "width=%d:height=%d:pix_fmt=%d:sar=%d/%d:"
+#if HB_PROJECT_FEATURE_QSV
+    if (hb_qsv_hw_filters_are_enabled(graph->job))
+    {
+        par = av_buffersrc_parameters_alloc();
+        filter_args = hb_strdup_printf(
+                "video_size=%dx%d:pix_fmt=%d:sar=%d/%d:"
                 "time_base=%d/%d:frame_rate=%d/%d",
-                init->geometry.width, init->geometry.height, init->pix_fmt,
+                init->geometry.width, init->geometry.height, AV_PIX_FMT_QSV,
                 init->geometry.par.num, init->geometry.par.den,
                 init->time_base.num, init->time_base.den,
                 init->vrate.num, init->vrate.den);
 
-    avfilter = append_filter(graph, "buffer", filter_args);
+        AVBufferRef *hb_hw_frames_ctx = NULL;
+
+        result = hb_create_ffmpeg_pool(graph->job, init->geometry.width, init->geometry.height, init->pix_fmt, 32, 0, &hb_hw_frames_ctx);
+        if (result < 0)
+        {
+            hb_error("hb_create_ffmpeg_pool failed");
+            goto fail;
+        }
+        par->hw_frames_ctx = hb_hw_frames_ctx;
+        // TODO: need to pass correct alignment for the scale_qsv output frame
+        int out_alignment = 0;
+        switch (graph->job->vcodec)
+        {
+            case HB_VCODEC_QSV_H264:
+                out_alignment = 16;
+                break;
+            case HB_VCODEC_QSV_H265_10BIT:
+            case HB_VCODEC_QSV_H265:
+                out_alignment = 32;
+                break;
+            default:
+                out_alignment = 0;
+                break;
+        }
+        graph->avgraph->opaque = (void*)(intptr_t)out_alignment;
+    }
+    else
+#endif
+    {
+        filter_args = hb_strdup_printf(
+                    "width=%d:height=%d:pix_fmt=%d:sar=%d/%d:"
+                    "time_base=%d/%d:frame_rate=%d/%d",
+                    init->geometry.width, init->geometry.height, init->pix_fmt,
+                    init->geometry.par.num, init->geometry.par.den,
+                    init->time_base.num, init->time_base.den,
+                    init->vrate.num, init->vrate.den);
+    }
+    avfilter = append_filter(graph, "buffer", filter_args, par);
+    av_free(par);
     free(filter_args);
     if (avfilter == NULL)
     {
@@ -120,7 +176,7 @@ hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
     graph->last = out->filter_ctx;
 
     // Build filter output
-    avfilter = append_filter(graph, "buffersink", NULL);
+    avfilter = append_filter(graph, "buffersink", NULL, 0);
     if (avfilter == NULL)
     {
         hb_error("hb_avfilter_graph_init: failed to create buffer output filter");
@@ -161,6 +217,7 @@ hb_avfilter_graph_init(hb_value_t * settings, hb_filter_init_t * init)
     return graph;
 
 fail:
+    av_free(par);
     avfilter_inout_free(&in);
     avfilter_inout_free(&out);
     hb_avfilter_graph_close(&graph);
@@ -224,8 +281,18 @@ int hb_avfilter_add_buf(hb_avfilter_graph_t * graph, hb_buffer_t * in)
 {
     if (in != NULL)
     {
-        hb_video_buffer_to_avframe(graph->frame, in);
-        return av_buffersrc_add_frame(graph->input, graph->frame);
+#if HB_PROJECT_FEATURE_QSV
+        if (hb_qsv_hw_filters_are_enabled(graph->job))
+        {
+            hb_video_buffer_to_avframe(in->qsv_details.frame, in);
+            return hb_avfilter_add_frame(graph, in->qsv_details.frame);
+        }
+        else
+#endif
+        {
+            hb_video_buffer_to_avframe(graph->frame, in);
+            return av_buffersrc_add_frame(graph->input, graph->frame);
+        }
     }
     else
     {
@@ -241,7 +308,17 @@ hb_buffer_t * hb_avfilter_get_buf(hb_avfilter_graph_t * graph)
     if (result >= 0)
     {
         hb_buffer_t * buf;
-        buf = hb_avframe_to_video_buffer(graph->frame, graph->out_time_base);
+#if HB_PROJECT_FEATURE_QSV
+        if (hb_qsv_hw_filters_are_enabled(graph->job))
+        {
+            buf = hb_qsv_copy_avframe_to_video_buffer(graph->job, graph->frame, 1);
+            hb_avframe_set_video_buffer_flags(buf, graph->frame, graph->out_time_base);
+        }
+        else
+#endif
+        {
+            buf = hb_avframe_to_video_buffer(graph->frame, graph->out_time_base);
+        }
         av_frame_unref(graph->frame);
         return buf;
     }
@@ -267,6 +344,8 @@ void hb_avfilter_combine( hb_list_t * list)
             case HB_FILTER_CROP_SCALE:
             case HB_FILTER_PAD:
             case HB_FILTER_ROTATE:
+            case HB_FILTER_COLORSPACE:
+            case HB_FILTER_FORMAT:
             {
                 settings = pv->avfilters;
             } break;
