@@ -36,8 +36,9 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "handbrake/qsv_filter.h"
 #include "handbrake/qsv_libav.h"
 #include "handbrake/qsv_memory.h"
+#include "handbrake/qsv_common.h"
 
-
+#if !HB_QSV_ONEVPL
 static int hb_qsv_filter_pre_init( hb_filter_object_t * filter,
                                hb_filter_init_t * init );
 static int hb_qsv_filter_pre_work( hb_filter_object_t * filter,
@@ -54,7 +55,6 @@ static int hb_qsv_filter_post_work( hb_filter_object_t * filter,
 static hb_filter_info_t * hb_qsv_filter_post_info(hb_filter_object_t * filter);
 static void hb_qsv_filter_post_close( hb_filter_object_t * filter );
 
-
 hb_filter_object_t hb_filter_qsv_pre =
 {
     .id            = HB_FILTER_QSV_PRE,
@@ -79,7 +79,6 @@ hb_filter_object_t hb_filter_qsv_post =
     .info          = hb_qsv_filter_post_info,
 };
 
-
 static int filter_pre_init( hb_qsv_context* qsv, hb_filter_private_t * pv ){
     mfxStatus sts = MFX_ERR_NONE;
     int i=0;
@@ -287,15 +286,15 @@ static int hb_qsv_filter_pre_init( hb_filter_object_t * filter,
     // PIX_FMT_NV12,      ///< planar YUV 4:2:0, 12bpp, 1 plane for Y and 1 plane for the UV components, which are interleaved (first byte U and the following byte V)
     pv->sws_context_from_nv12 = hb_sws_get_context(
         pv->job->title->geometry.width, pv->job->title->geometry.height,
-        AV_PIX_FMT_NV12,
+        AV_PIX_FMT_NV12, pv->job->color_range,
         pv->job->title->geometry.width, pv->job->title->geometry.height,
-        AV_PIX_FMT_YUV420P,
+        AV_PIX_FMT_YUV420P, pv->job->color_range,
         SWS_LANCZOS|SWS_ACCURATE_RND, SWS_CS_DEFAULT);
     pv->sws_context_to_nv12 = hb_sws_get_context(
         pv->job->title->geometry.width, pv->job->title->geometry.height,
-        AV_PIX_FMT_YUV420P,
+        AV_PIX_FMT_YUV420P, pv->job->color_range,
         pv->job->title->geometry.width, pv->job->title->geometry.height,
-        AV_PIX_FMT_NV12,
+        AV_PIX_FMT_NV12, pv->job->color_range,
         SWS_LANCZOS|SWS_ACCURATE_RND, SWS_CS_DEFAULT);
     return 0;
 }
@@ -519,8 +518,8 @@ static void hb_qsv_filter_pre_close( hb_filter_object_t * filter ){
         // closing local stuff
         qsv_filter_close(qsv,HB_QSV_VPP_USER);
 
-        // closing the commong stuff
-        hb_qsv_context_clean(qsv);
+        // closing the common stuff
+        hb_qsv_context_clean(qsv,hb_qsv_full_path_is_enabled(pv->job));
     }
     hb_cond_close(&pv->pre.frame_completed);
     hb_lock_close(&pv->pre.frame_completed_lock);
@@ -815,6 +814,7 @@ mfxStatus plugin_close(qsv_filter_t* plugin){
 
     return sts;
 }
+#endif
 
 mfxExtBuffer* get_ext_buffer(mfxExtBuffer** buffers, mfxU32 buffers_num, mfxU32 buffer_id){
     int i = 0;
@@ -878,9 +878,9 @@ int process_filter(qsv_filter_task_t* task, void* params){
         unlock_frame(task->processor.alloc,task->in);
         return sts;
     }
-
+#if !HB_QSV_ONEVPL
     qsv_nv12_to_yuv420(task->pv->sws_context_from_nv12,task->pv->pre.out, task->in, task->processor.core);
-
+#endif
     // signal: input is prepared, converted from pipeline into internal buffer
     hb_lock(task->pv->pre.frame_completed_lock);
     task->pv->pre.frame_go = 1;
