@@ -207,7 +207,7 @@ void hb_qsv_add_context_usage(hb_qsv_context * qsv, int is_threaded)
     }
 }
 
-int hb_qsv_context_clean(hb_qsv_context * qsv)
+int hb_qsv_context_clean(hb_qsv_context * qsv, int full_job)
 {
     int is_active = 0;
     mfxStatus sts = MFX_ERR_NONE;
@@ -235,10 +235,12 @@ int hb_qsv_context_clean(hb_qsv_context * qsv)
         if (qsv->pipes)
             hb_qsv_pipe_list_clean(&qsv->pipes);
 
-        if (qsv->mfx_session) {
-            // Intentionally leave it because the decode session closed first and encoder session hangs
-            //sts = MFXClose(qsv->mfx_session);
-            //HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
+        if (qsv->mfx_session && !full_job) {
+            // MFXClose() fails in the media_driver under Linux when encoding interrupted
+#if defined(_WIN32) || defined(__MINGW32__)
+            sts = MFXClose(qsv->mfx_session);
+            HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
+#endif
             qsv->mfx_session = 0;
         }
     }
@@ -585,7 +587,7 @@ int av_is_qsv_available(mfxIMPL impl, mfxVersion * ver)
     return sts;
 }
 
-void hb_qsv_wait_on_sync(hb_qsv_context *qsv, hb_qsv_stage *stage)
+int hb_qsv_wait_on_sync(hb_qsv_context *qsv, hb_qsv_stage *stage)
 {
     int iter = 0;
     mfxStatus sts = MFX_ERR_NONE;
@@ -602,10 +604,11 @@ void hb_qsv_wait_on_sync(hb_qsv_context *qsv, hb_qsv_stage *stage)
                     hb_qsv_sleep(10);
                     continue;
                 }
-                HB_QSV_CHECK_RET(sts, MFX_ERR_NONE, sts);
+                HB_QSV_CHECK_RESULT(sts, MFX_ERR_NONE, sts);
                 break;
             }
         }
+    return 0;
 }
 
 #endif // HB_PROJECT_FEATURE_QSV
