@@ -1,6 +1,6 @@
 /* reader.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -279,6 +279,13 @@ static int reader_init( hb_work_object_t * w, hb_job_t * job )
     else
     {
         int count = hb_list_count(job->title->list_chapter);
+
+        if (job->chapter_end > count || job->chapter_start > count || job->chapter_start > job->chapter_end)
+        {
+            hb_error("Invalid chapter start/end indexes");
+            return 1;
+        }
+
         if (count == 0 || count <= job->chapter_end)
         {
             r->duration = job->title->duration;
@@ -315,7 +322,7 @@ static int reader_init( hb_work_object_t * w, hb_job_t * job )
     // fifos that will be needed (+1 for null terminator)
     r->fifos = calloc(count + 1, sizeof(hb_fifo_t*));
 
-    // The stream needs to be open before starting the reader thead
+    // The stream needs to be open before starting the reader thread
     // to prevent a race with decoders that may share information
     // with the reader. Specifically avcodec needs this.
     if ( hb_reader_open( r ) )
