@@ -1,6 +1,6 @@
 /* work.c
 
-   Copyright (c) 2003-2021 HandBrake Team
+   Copyright (c) 2003-2022 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -294,14 +294,10 @@ hb_work_object_t* hb_video_encoder(hb_handle_t *h, int vcodec)
             break;
 #endif
 #ifdef __APPLE__
-        case HB_VCODEC_FFMPEG_VT_H264:
-            w = hb_get_work(h, WORK_ENCAVCODEC);
-            w->codec_param = AV_CODEC_ID_H264;
-            break;
-        case HB_VCODEC_FFMPEG_VT_H265:
-        case HB_VCODEC_FFMPEG_VT_H265_10BIT:
-            w = hb_get_work(h, WORK_ENCAVCODEC);
-            w->codec_param = AV_CODEC_ID_HEVC;
+        case HB_VCODEC_VT_H264:
+        case HB_VCODEC_VT_H265:
+        case HB_VCODEC_VT_H265_10BIT:
+            w = hb_get_work(h, WORK_ENCVT);
             break;
 #endif
 #if HB_PROJECT_FEATURE_MF
@@ -554,9 +550,9 @@ void hb_display_job_info(hb_job_t *job)
                 case HB_VCODEC_FFMPEG_VCE_H265:
                 case HB_VCODEC_FFMPEG_NVENC_H264:
                 case HB_VCODEC_FFMPEG_NVENC_H265:
-                case HB_VCODEC_FFMPEG_VT_H264:
-                case HB_VCODEC_FFMPEG_VT_H265:
-                case HB_VCODEC_FFMPEG_VT_H265_10BIT:
+                case HB_VCODEC_VT_H264:
+                case HB_VCODEC_VT_H265:
+                case HB_VCODEC_VT_H265_10BIT:
                 case HB_VCODEC_FFMPEG_MF_H264:
                 case HB_VCODEC_FFMPEG_MF_H265:
                     hb_log("     + profile: %s", job->encoder_profile);
@@ -580,9 +576,9 @@ void hb_display_job_info(hb_job_t *job)
                 case HB_VCODEC_FFMPEG_VCE_H265:
                 case HB_VCODEC_FFMPEG_NVENC_H264:
                 case HB_VCODEC_FFMPEG_NVENC_H265:
-                case HB_VCODEC_FFMPEG_VT_H264:
+                case HB_VCODEC_VT_H264:
                 // VT h.265 currently only supports auto level
-                // case HB_VCODEC_FFMPEG_VT_H265:
+                // case HB_VCODEC_VT_H265:
                 // MF h.264/h.265 currently only supports auto level
                 // case HB_VCODEC_FFMPEG_MF_H264:
                 // case HB_VCODEC_FFMPEG_MF_H265:
@@ -616,6 +612,9 @@ void hb_display_job_info(hb_job_t *job)
 
         hb_log("     + color profile: %d-%d-%d",
                job->color_prim, job->color_transfer, job->color_matrix);
+        hb_log("     + chroma location: %s",
+               av_chroma_location_name(job->chroma_location));
+
 
         if (job->color_transfer == HB_COLR_TRA_SMPTEST2084)
         {
@@ -1461,11 +1460,12 @@ static void do_job(hb_job_t *job)
         init.time_base.den = 90000;
         init.job = job;
         init.pix_fmt = job->input_pix_fmt;
-        init.color_range = AVCOL_RANGE_MPEG;
 
         init.color_prim = title->color_prim;
         init.color_transfer = title->color_transfer;
         init.color_matrix = title->color_matrix;
+        init.color_range = AVCOL_RANGE_MPEG;
+        init.chroma_location = title->chroma_location;
         init.geometry = title->geometry;
         memset(init.crop, 0, sizeof(int[4]));
         init.vrate = job->vrate;
@@ -1491,6 +1491,7 @@ static void do_job(hb_job_t *job)
         job->color_transfer = init.color_transfer;
         job->color_matrix = init.color_matrix;
         job->color_range = init.color_range;
+        job->chroma_location = init.chroma_location;
         job->width = init.geometry.width;
         job->height = init.geometry.height;
         // job->par is supplied by the frontend.
@@ -1884,7 +1885,9 @@ cleanup:
     }
     hb_buffer_pool_free();
 #if HB_PROJECT_FEATURE_QSV
-    if (!job->indepth_scan && hb_qsv_is_enabled(job))
+    if (!job->indepth_scan &&
+        (job->pass_id != HB_PASS_ENCODE_1ST) &&
+        hb_qsv_is_enabled(job))
     {
         hb_qsv_context_uninit(job);
     }
