@@ -528,6 +528,12 @@ class BuildTupleProbe( ShellProbe, list ):
         self.release = self[3]
         self.extra   = self[4]
 
+        ## special mapping for Apple Silicon
+        ## config.guess returns aarch64, we need arm64
+        if self.vendor == 'apple' and self.system == 'darwin':
+            if self.machine == 'aarch64':
+                self[0] = self.machine = 'arm64'
+
         ## nice formal name for 'system'
         self.systemf = platform.system()
 
@@ -544,15 +550,26 @@ class BuildTupleProbe( ShellProbe, list ):
 ###############################################################################
 
 class HostTupleAction( Action, list ):
-    def __init__( self, cross=None ):
+    def __init__( self, cross=None, arch_gcc=None, xcode_opts=None ):
         super( HostTupleAction, self ).__init__( 'compute', 'host tuple', abort=True )
         # Initialize, but allow to be reset by options
-        self.setHost(cross)
+        self.setHost(cross, arch_gcc, xcode_opts)
 
-    def setHost( self, cross ):
+    def setHost( self, cross=None, arch_gcc=None, xcode_opts=None ):
         ## check if --cross spec was used; must maintain 5-tuple compatibility with regex
+
+        ## special mapping for Apple Silicon
+        ## config.guess returns aarch64, we need arm64
+        if build_tuple.vendor == 'apple' and build_tuple.system == 'darwin':
+            if build_tuple.spec.startswith('aarch64'):
+                build_tuple.spec = 'arm64' + build_tuple.spec.lstrip('aarch64')
+
         if cross is not None:
             self.spec = os.path.basename( cross ).rstrip( '-' )
+        elif arch_gcc is not None:
+            self.spec = arch_gcc + build_tuple.spec.lstrip(build_tuple.machine)
+        elif xcode_opts is not None and xcode_opts['config'] is not None and not xcode_opts['disabled']:
+            self.spec = xcode_opts['config'].split(".")[-1] + build_tuple.spec.lstrip(build_tuple.machine)
         else:
             self.spec = build_tuple.spec
 
@@ -595,9 +612,9 @@ class HostTupleAction( Action, list ):
 
     def _action( self ):
         try:
-            self.setHost(options.cross)
+            self.setHost(options.cross,arch_gcc,xcode_opts)
         except NameError:
-            self.setHost(None)
+            self.setHost()
 
     ## glob-match against spec
     def match( self, *specs ):
@@ -674,7 +691,8 @@ class ArchAction( Action ):
         elif host_tuple.match( '*-*-mingw*' ):
             pass
         elif host_tuple.match( '*-*-darwin*' ):
-            self.mode['x86_64'] = 'x86_64-apple-darwin%s'    % (host_tuple.release)
+            self.mode['arm64']  =  'arm64-apple-darwin%s' % (host_tuple.release)
+            self.mode['x86_64'] = 'x86_64-apple-darwin%s' % (host_tuple.release)
         elif host_tuple.match( '*-*-linux*' ):
             pass
         elif host_tuple.match( '*-*-solaris*' ):
@@ -863,7 +881,11 @@ class RepoProbe( ShellProbe ):
 
             elif name == 'HASH':
                 self.hash = value
-                self.shorthash = value[:7]
+            elif name == 'SHORTHASH':
+                if value != '':
+                    self.shorthash = value
+                else:
+                    self.shorthash = self.hash
 
         # type-classification via repository URL
         if self.url == project.url_repo_ssh:
@@ -1088,8 +1110,9 @@ class VersionProbe( Action ):
             self.rexprs.insert(0,rexpr)
 
     def _action( self ):
-        ## pipe and redirect stderr to stdout; effects communicate result
-        pipe = subprocess.Popen( self.command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
+        with open(os.devnull, 'w') as devnull:
+            ## pipe and redirect stderr to dev/null; effects communicate result
+            pipe = subprocess.Popen( self.command, stdout=subprocess.PIPE, stderr=devnull )
 
         ## read data into memory buffers
         data = pipe.communicate()
@@ -1353,9 +1376,12 @@ def createCLI( cross = None ):
 
     ## add security options
     grp = cli.add_argument_group( 'Security Options' )
-    grp.add_argument( '--harden', dest="enable_harden", default=False, action='store_true', help='harden app to protect against buffer overflows' )
-    h = IfHost( 'sandbox app to limit host system access (macOS only)', '*-*-darwin*', none=argparse.SUPPRESS).value
-    grp.add_argument( '--sandbox', dest="enable_sandbox", default=False, action='store_true', help=(( '%s' %h ) if h != argparse.SUPPRESS else h) )
+    h = IfHost( 'hardening to protect against buffer overflows', '*-*-*', none=argparse.SUPPRESS).value
+    grp.add_argument( '--harden', dest="enable_harden", default=True, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    grp.add_argument( '--no-harden', dest="enable_harden", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
+    h = IfHost( 'sandboxing to limit host system access (macOS only)', '*-*-darwin*', none=argparse.SUPPRESS).value
+    grp.add_argument( '--sandbox', dest="enable_sandbox", default=True, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    grp.add_argument( '--no-sandbox', dest="enable_sandbox", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
     cli.add_argument_group( grp )
 
     ## add Xcode options
@@ -1378,10 +1404,10 @@ def createCLI( cross = None ):
     h = IfHost( 'enable assembly code in non-contrib modules', 'NOMATCH*-*-darwin*', 'NOMATCH*-*-linux*', none=argparse.SUPPRESS ).value
     grp.add_argument( '--enable-asm', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable GTK GUI', '*-*-linux*', '*-*-freebsd*', none=argparse.SUPPRESS ).value
+    h = IfHost( 'disable GTK GUI', '*-*-linux*', '*-*-freebsd*', '*-*-netbsd*', none=argparse.SUPPRESS ).value
     grp.add_argument( '--disable-gtk', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', '*-*-freebsd*', none=argparse.SUPPRESS ).value
+    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', '*-*-freebsd*', '*-*-netbsd*', none=argparse.SUPPRESS ).value
     grp.add_argument( '--disable-gtk-update-checks', default=False, action='store_true', help=h )
 
     h = 'enable GTK GUI for Windows' if (cross is not None and 'mingw' in cross) else argparse.SUPPRESS
@@ -1390,7 +1416,7 @@ def createCLI( cross = None ):
     h = IfHost( 'Build GUI with GTK4', '*-*-linux*', '*-*-freebsd*', none=argparse.SUPPRESS ).value
     grp.add_argument( '--enable-gtk4', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable GStreamer (live preview)', '*-*-linux*', '*-*-freebsd*', none=argparse.SUPPRESS ).value
+    h = IfHost( 'disable GStreamer (live preview)', '*-*-linux*', '*-*-freebsd*', '*-*-netbsd*', none=argparse.SUPPRESS ).value
     grp.add_argument( '--disable-gst', default=False, action='store_true', help=h )
 
     h = IfHost( 'x265 video encoder', '*-*-*', none=argparse.SUPPRESS ).value
@@ -1409,16 +1435,20 @@ def createCLI( cross = None ):
     grp.add_argument( '--enable-ffmpeg-aac', dest="enable_ffmpeg_aac", default=not host_tuple.match( '*-*-darwin*' ), action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
     grp.add_argument( '--disable-ffmpeg-aac', dest="enable_ffmpeg_aac", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
 
-    h = IfHost( 'Nvidia NVENC video encoder', '*-*-linux*', '*-*-mingw*', none=argparse.SUPPRESS).value
-    grp.add_argument( '--enable-nvenc', dest="enable_nvenc", default=IfHost( True, '*-*-linux*', '*-*-mingw*', none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    h = IfHost( 'MediaFoundation video encoder', 'aarch64-w64-mingw32', none=argparse.SUPPRESS).value
+    grp.add_argument( '--enable-mf', dest="enable_mf", default=False, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    grp.add_argument( '--disable-mf', dest="enable_mf", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
+
+    h = IfHost( 'Nvidia NVENC video encoder', '*-*-linux*', 'x86_64-w64-mingw32', none=argparse.SUPPRESS).value
+    grp.add_argument( '--enable-nvenc', dest="enable_nvenc", default=IfHost( True, '*-*-linux*', 'x86_64-w64-mingw32', none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
     grp.add_argument( '--disable-nvenc', dest="enable_nvenc", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
 
-    h = IfHost( 'Intel QSV video encoder/decoder', '*-*-linux*', '*-*-mingw*', none=argparse.SUPPRESS).value
-    grp.add_argument( '--enable-qsv', dest="enable_qsv", default=IfHost(True, "*-*-mingw*", none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    h = IfHost( 'Intel QSV video encoder/decoder', '*-*-linux*', '*-*-freebsd*', 'x86_64-w64-mingw32', none=argparse.SUPPRESS).value
+    grp.add_argument( '--enable-qsv', dest="enable_qsv", default=IfHost(True, "x86_64-w64-mingw32", none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
     grp.add_argument( '--disable-qsv', dest="enable_qsv", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
 
-    h = IfHost( 'AMD VCE video encoder', '*-*-linux*', '*-*-mingw*', none=argparse.SUPPRESS).value
-    grp.add_argument( '--enable-vce', dest="enable_vce", default=IfHost(True, '*-*-mingw*', none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
+    h = IfHost( 'AMD VCE video encoder', '*-*-linux*', 'x86_64-w64-mingw32', none=argparse.SUPPRESS).value
+    grp.add_argument( '--enable-vce', dest="enable_vce", default=IfHost(True, 'x86_64-w64-mingw32', none=False).value, action='store_true', help=(( 'enable %s' %h ) if h != argparse.SUPPRESS else h) )
     grp.add_argument( '--disable-vce', dest="enable_vce", action='store_false', help=(( 'disable %s' %h ) if h != argparse.SUPPRESS else h) )
 
     cli.add_argument_group( grp )
@@ -1561,20 +1591,38 @@ try:
     project = Project()
 
     # options is created by parse_known_args(), which is called directly after
-    # createCLI(). we need cross info earlier and cannot parse args twice, so
-    # extract the info we need here
-    cross = None
+    # createCLI(). we need some options info earlier and cannot parse args
+    # twice, so extract the info we need here from sys.argv
+    arch_gcc = None
+    cross    = None
+    xcode_opts = { 'disabled': False, 'config': None }
     for i in range(len(sys.argv)):
-        cross_pattern = re.compile( '^--cross=(.+)$' )
-        m = cross_pattern.match( sys.argv[i] )
-        if m:
+        if re.compile( '^--arch=(.+)$' ).match( sys.argv[i] ):
+            arch_gcc = sys.argv[i][7:]
+            continue
+        elif re.compile( '^--arch$' ).match( sys.argv[i] ) and ((i + 1) < len(sys.argv)):
+            arch_gcc = sys.argv[i+1]
+            arch_gcc = None if arch_gcc == '' else arch_gcc
+            i = i + 1
+            continue
+        elif re.compile( '^--cross=(.+)$' ).match( sys.argv[i] ):
             cross = sys.argv[i][8:]
             continue
-        cross_pattern = re.compile( '^--cross$' )
-        m = cross_pattern.match( sys.argv[i] )
-        if m and ((i + 1) < len(sys.argv)):
+        elif re.compile( '^--cross$' ).match( sys.argv[i] ) and ((i + 1) < len(sys.argv)):
             cross = sys.argv[i+1]
             cross = None if cross == '' else cross
+            i = i + 1
+            continue
+        elif re.compile( '^--xcode-config=(.+)$' ).match( sys.argv[i] ):
+            xcode_opts['config'] = sys.argv[i][15:]
+            continue
+        elif re.compile( '^--xcode-config$' ).match( sys.argv[i] ) and ((i + 1) < len(sys.argv)):
+            xcode_opts['config'] = sys.argv[i+1]
+            xcode_opts['config'] = None if xcode_opts['config'] == '' else xcode_opts['config']
+            i = i + 1
+            continue
+        elif re.compile( '^--disable-xcode$' ).match( sys.argv[i] ):
+            xcode_opts['disabled'] = True
             continue
 
     ## create tools in a scope
@@ -1602,7 +1650,7 @@ try:
         else:
             gmake  = ToolProbe( 'GMAKE.exe',      'make',       'gmake', 'make', abort=True )
 
-        autoconf   = ToolProbe( 'AUTOCONF.exe',   'autoconf',   'autoconf', abort=True, minversion=[2,69,0] )
+        autoconf   = ToolProbe( 'AUTOCONF.exe',   'autoconf',   'autoconf', abort=True, minversion=([2,71,0] if build_tuple.match('*-*-darwin*') else [2,69,0]) )
         automake   = ToolProbe( 'AUTOMAKE.exe',   'automake',   'automake', abort=True, minversion=[1,13,0] )
         libtool    = ToolProbe( 'LIBTOOL.exe',    'libtool',    'libtool', abort=True )
         lipo       = ToolProbe( 'LIPO.exe',       'lipo',       'lipo', abort=False )
@@ -1611,15 +1659,12 @@ try:
         nasm       = ToolProbe( 'NASM.exe',       'asm',        'nasm', abort=True, minversion=[2,13,0] )
         ninja      = ToolProbe( 'NINJA.exe',      'ninja',      'ninja-build', 'ninja', abort=True )
 
-        xcodebuild = ToolProbe( 'XCODEBUILD.exe', 'xcodebuild', 'xcodebuild', abort=(True if (build_tuple.match('*-*-darwin*') and cross is None) else False), versionopt='-version', minversion=[10,3,0] )
+        xcodebuild = ToolProbe( 'XCODEBUILD.exe', 'xcodebuild', 'xcodebuild', abort=(True if (not xcode_opts['disabled'] and (build_tuple.match('*-*-darwin*') and cross is None)) else False), versionopt='-version', minversion=[10,3,0] )
 
     ## run tool probes
     for tool in ToolProbe.tools:
         tool.run()
 
-    debugMode = SelectMode( 'debug', ('none','none'), ('min','min'), ('std','std'), ('max','max') )
-    optimizeMode = SelectMode( 'optimize', ('none','none'), ('speed','speed'), ('size','size'), default='speed' )
-
     ## find xcconfig values
     xcconfigMode = SelectMode( 'xcconfig', ('none',None), what='' )
     if build_tuple.match( '*-*-darwin*' ):
@@ -1639,8 +1684,18 @@ try:
             tool.__init__( tool.var, tool.option, '%s-%s' % (cross,tool.name), **tool.kwargs )
             tool.run()
 
+    debugMode = SelectMode( 'debug', ('none','none'), ('min','min'), ('std','std'), ('max','max') )
+
+    Oz_check_command = '%s -Oz -S -o /dev/null -xc /dev/null > /dev/null 2>&1' % Tools.gcc.pathname
+    Oz_check = ShellProbe('checking for -Oz', '%s' % Oz_check_command)
+    Oz_check.run()
+    if Oz_check.fail is False:
+        optimizeMode = SelectMode( 'optimize', ('none','none'), ('speed','speed'), ('size','size'), ('size-aggressive','size-aggressive'), default='speed' )
+    else:
+        optimizeMode = SelectMode( 'optimize', ('none','none'), ('speed','speed'), ('size','size'), default='speed' )
+
     # run host tuple and arch actions
-    host_tuple = HostTupleAction(cross)
+    host_tuple = HostTupleAction(cross,arch_gcc,xcode_opts)
     arch       = ArchAction(); arch.run()
 
     # create CLI and parse
@@ -1675,28 +1730,25 @@ try:
     # Allow GTK mingw only on mingw
     options.enable_gtk_mingw  = IfHost(options.enable_gtk_mingw, '*-*-mingw*',
                                        none=False).value
-    # Disable NVENC on unsupported platforms
-    options.enable_nvenc      = IfHost(options.enable_nvenc, '*-*-linux*',
-                                       '*-*-mingw*', none=False).value
     # NUMA is linux only and only needed with x265
     options.enable_numa       = (IfHost(options.enable_numa, '*-*-linux*',
                                         none=False).value
                                  and options.enable_x265)
-    # Disable QSV on unsupported platforms
-    options.enable_qsv        = IfHost(options.enable_qsv, '*-*-linux*',
-                                       '*-*-mingw*', none=False).value
-    # Disable VCE on unsupported platforms
-    options.enable_vce        = IfHost(options.enable_vce, '*-*-linux*', '*-*-mingw*',
+    # Only allow these features on supported platforms
+    options.enable_mf         = IfHost(options.enable_mf, 'aarch64-w64-mingw32',
                                        none=False).value
+    options.enable_nvenc      = IfHost(options.enable_nvenc, '*-*-linux*',
+                                       'x86_64-w64-mingw32', none=False).value
+    options.enable_qsv        = IfHost(options.enable_qsv, '*-*-linux*', '*-*-freebsd*',
+                                       'x86_64-w64-mingw32', none=False).value
+    options.enable_vce        = IfHost(options.enable_vce, '*-*-linux*',
+                                       'x86_64-w64-mingw32', none=False).value
 
     #####################################
     ## Additional library and tool checks
     #####################################
-    if options.flatpak or host_tuple.match('*-*-darwin*', '*-*-mingw'):
-        # Requires Jansson which requires CMake 3.1.0 or later
-        Tools.cmake = ToolProbe('CMAKE.exe', 'cmake', 'cmake', abort=True, minversion=[3,1,0])
-    else:
-        Tools.cmake = ToolProbe('CMAKE.exe', 'cmake', 'cmake', abort=True, minversion=[2,8,12])
+    # Requires oneVPL which requires CMake 3.16.3 or later
+    Tools.cmake = ToolProbe('CMAKE.exe', 'cmake', 'cmake', abort=True, minversion=[3,16,3])
     Tools.cmake.__init__( Tools.cmake.var, Tools.cmake.option, Tools.cmake.name, **Tools.cmake.kwargs )
     Tools.cmake.run()
     for action in Action.actions:
@@ -1965,7 +2017,7 @@ int main()
     doc.add( 'HOST.extra',   host_tuple.extra )
     doc.add( 'HOST.title',   host_tuple.title )
 
-    doc.add( 'HOST.cross', int(options.cross != None or arch.mode.mode != arch.mode.default) )
+    doc.add( 'HOST.cross', int(options.cross != None or arch.mode.mode != arch.mode.default or build_tuple.machine != host_tuple.machine) )
     if options.cross:
         doc.add( 'HOST.cross.prefix', '%s-' % (options.cross) )
     else:
@@ -1995,6 +2047,7 @@ int main()
     doc.add( 'FEATURE.gtk.mingw',  int( options.enable_gtk_mingw ))
     doc.add( 'FEATURE.gtk.update.checks', int( not options.disable_gtk_update_checks ))
     doc.add( 'FEATURE.gst',        int( not options.disable_gst ))
+    doc.add( 'FEATURE.mf',         int( options.enable_mf ))
     doc.add( 'FEATURE.nvenc',      int( options.enable_nvenc ))
     doc.add( 'FEATURE.qsv',        int( options.enable_qsv ))
     doc.add( 'FEATURE.vce',        int( options.enable_vce ))
@@ -2018,7 +2071,7 @@ int main()
         doc.addBlank()
         if not dlfcn.fail:
             doc.add( 'HAS.dlfcn', 1 )
-        elif not pthread.fail:
+        if not pthread.fail:
             doc.add( 'HAS.pthread', 1 )
         if not bz2.fail:
             doc.add( 'HAS.bz2', 1 )
@@ -2035,7 +2088,7 @@ int main()
 
     else:
         doc.addBlank()
-        if host_tuple.system == 'freebsd':
+        if host_tuple.system in ('freebsd', 'netbsd'):
             doc.add( 'HAS.pthread', 1 )
         if not strerror_r.fail:
             doc.add( 'HAS.strerror_r', 1 )
@@ -2113,19 +2166,21 @@ int main()
     stdout.write( 'Build system:       %s\n' % build_tuple.spec.rstrip('-') )
     stdout.write( 'Host system:        %s\n' % host_tuple.spec.rstrip('-') )
     stdout.write( 'Target platform:    %s' % host_tuple.system )
-    stdout.write( ' (cross-compile)\n' ) if options.cross else stdout.write( '\n' )
+    stdout.write( ' (cross-compile)\n' ) if options.cross or build_tuple.machine != host_tuple.machine else stdout.write( '\n' )
     stdout.write( 'Harden:             %s\n' % options.enable_harden )
     stdout.write( 'Sandbox:            %s' % options.enable_sandbox )
     stdout.write( ' (%s)\n' % note_unsupported ) if not host_tuple.system == 'darwin' else stdout.write( '\n' )
     stdout.write( 'Enable FDK-AAC:     %s\n' % options.enable_fdk_aac )
     stdout.write( 'Enable FFmpeg AAC:  %s' % options.enable_ffmpeg_aac )
     stdout.write( '  (%s)\n' % note_required ) if host_tuple.system != 'darwin' else stdout.write( '\n' )
+    stdout.write( 'Enable MediaFound.: %s' % options.enable_mf )
+    stdout.write( ' (%s)\n' % note_unsupported ) if not host_tuple.match( 'aarch64-w64-mingw32' ) else stdout.write( '\n' )
     stdout.write( 'Enable NVENC:       %s' % options.enable_nvenc )
-    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.system == 'mingw') else stdout.write( '\n' )
+    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.match( 'x86_64-w64-mingw32' )) else stdout.write( '\n' )
     stdout.write( 'Enable QSV:         %s' % options.enable_qsv )
-    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.system == 'mingw') else stdout.write( '\n' )
+    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.match( 'x86_64-w64-mingw32' ) or host_tuple.system == 'freebsd') else stdout.write( '\n' )
     stdout.write( 'Enable VCE:         %s' % options.enable_vce )
-    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.system == 'mingw') else stdout.write( '\n' )
+    stdout.write( ' (%s)\n' % note_unsupported ) if not (host_tuple.system == 'linux' or host_tuple.match( 'x86_64-w64-mingw32' )) else stdout.write( '\n' )
 
     if options.launch:
         stdout.write( '%s\n' % ('-' * 79) )
