@@ -88,7 +88,7 @@ class FlatpakManifest:
 
             if value.entry_type == SourceType.archive:
                 if handbrake_found:
-                    print "Error: only one archive source permitted"
+                    print("Error: only one archive source permitted")
                     sys.exit(3)
 
                 source["type"] = "archive"
@@ -104,15 +104,15 @@ class FlatpakManifest:
 
 
 def usage():
-    print "create_flatpak_manifest [-a <archive>] [-c <contrib>] [-s <sha265>] [-t <template>] [-r <sdk-runtime-version] [-h] [<dst>]"
-    print "     -a --archive    - Main archive (a.k.a. HB sources)"
-    print "     -c --contrib    - Contrib download URL (can be repeated)"
-    print "     -s --sha256     - sha256 of previous file on command line"
-    print "     -t --template   - Flatpak manifest template"
-    print "     -r --runtime    - Flatpak SDK runtime version"
-    print "     -q --qsv        - Build with Intel QSV support"
-    print "     -p --plugin     - Manifest if for a HandBrake flatpak plugin"
-    print "     -h --help       - Show this message"
+    print("create_flatpak_manifest [-a <archive>] [-c <contrib>] [-s <sha265>] [-t <template>] [-r <sdk-runtime-version] [-h] [<dst>]")
+    print("     -a --archive    - Main archive (a.k.a. HB sources)")
+    print("     -c --contrib    - Contrib download URL (can be repeated)")
+    print("     -s --sha256     - sha256 of previous file on command line")
+    print("     -t --template   - Flatpak manifest template")
+    print("     -r --runtime    - Flatpak SDK runtime version")
+    print("     -q --qsv        - Build with Intel QSV support")
+    print("     -p --plugin     - Manifest if for a HandBrake flatpak plugin")
+    print("     -h --help       - Show this message")
 
 if __name__ == "__main__":
     try:
@@ -120,7 +120,7 @@ if __name__ == "__main__":
             ["archive=", "contrib=", "sha265=",
              "template=", "runtime=", "qsv", "plugin", "help"])
     except getopt.GetoptError:
-        print "Error: Invalid option"
+        print("Error: Invalid option")
         usage()
         sys.exit(2)
 
@@ -175,5 +175,5 @@ if __name__ == "__main__":
         with open(dst, 'w') as fp:
             json.dump(manifest.manifest, fp, ensure_ascii=False, indent=4)
     else:
-        print json.dumps(manifest.manifest, ensure_ascii=False, indent=4)
+        print(json.dumps(manifest.manifest, ensure_ascii=False, indent=4))
 
